/*--------------------------------------------------------------------*\
  File:      KThread.cpp
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1998, Matt Bogosian. All rights reserved.
  Description: Source file containing the member functions for the
      KThread class.
  ID:        $Id: KThread.cpp,v 1.19 1998/07/01 05:16:16 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "KThread.h"


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

// Initialize the KThread private static const data members
const char KThread::mk_default_name[B_OS_NAME_LENGTH] = "KThread";


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-= Function Definitions =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

//====================================================================
KThread::KThread(const char a_name[B_OS_NAME_LENGTH], const int32 a_priority, status_t * const a_err) :
//====================================================================
	m_thread(spawn_thread(static_cast<thread_func>(ThreadFunc), a_name, a_priority, this)),
	m_lock(a_name)
{
	// Give the error back to the caller
	if (a_err != NULL)
	{
		*a_err = KThread::Status();
	}
}

//====================================================================
KThread::~KThread(void)
//====================================================================
{
	if (m_thread >= B_NO_ERROR)
	{
		kill_thread(m_thread);
	}
}

//====================================================================
void KThread::Kill(void)
//====================================================================
{
	thread_id thread(B_BAD_THREAD_ID);
	
	// Invalidate the thread
	if (Lock() == B_NO_ERROR)
	{
		thread = m_thread;
		m_thread = B_BAD_THREAD_ID;
		Unlock();
	}
	
	// Kill it
	if (thread != B_BAD_THREAD_ID)
	{
		kill_thread(thread);
	}
}

//====================================================================
status_t KThread::Status(void) const
//====================================================================
{
	status_t err;
	
	if ((err = m_lock.Status()) == B_NO_ERROR)
	{
		err = (m_thread >= B_NO_ERROR) ? B_NO_ERROR : m_thread;
	}
	
	return err;
}

//====================================================================
status_t KThread::Receive(int32 &a_msg_code, thread_id * const a_thread, void * const a_buf, const size_t a_buf_size)
//====================================================================
{
	// Try to receive the data
	int32 msg_code;
	thread_id thread;
	
	if ((msg_code = receive_data(&thread, a_buf, a_buf_size)) == B_INTERRUPTED)
	{
		return B_INTERRUPTED;
	}
	
	// Give the thread to the caller
	if (a_thread != NULL)
	{
		*a_thread = thread;
	}
	
	// The the message code to the caller
	a_msg_code = msg_code;
	
	return B_NO_ERROR;
}
